from unittest.mock import patch

import pytest

from ballet.eng.misc import IdentityTransformer
from ballet.feature import Feature
from ballet.validation.feature_acceptance.validator import (
    GFSSFAccepter, NeverAccepter, RandomAccepter,)
from tests.util import load_regression_data


def test_noop_accepter():
    X_df = y_df = y = None
    existing_features = []
    feature = None

    expected = False

    accepter = NeverAccepter(X_df, y_df, y, existing_features, feature)
    actual = accepter.judge()

    assert expected == actual


@patch('random.uniform', return_value=0.0)  # makes sure feature is accepted
def test_random_accepter(mock_uniform):
    X_df = y_df = y = None
    existing_features = []
    candidate_feature = None

    expected = True

    accepter = RandomAccepter(
        X_df, y_df, y, existing_features, candidate_feature)
    actual = accepter.judge()

    assert expected == actual


@pytest.fixture
def sample_data():
    X_df, y_df = load_regression_data(
        n_informative=1, n_uninformative=14)
    y = y_df
    return X_df, y_df, y


def test_gfssf_accepter_init(sample_data):
    X_df, y_df, y = sample_data

    feature_1 = Feature(
        input='A_0',
        transformer=IdentityTransformer(),
        source='1st Feature')
    feature_2 = Feature(
        input='Z_0',
        transformer=IdentityTransformer(),
        source='2nd Feature')

    features = [feature_1]
    candidate_feature = feature_2

    accepter = GFSSFAccepter(
        X_df, y_df, y, features, candidate_feature)

    assert accepter is not None



from typing import NamedTuple

import numpy as np
import pandas as pd
import pytest


class SampleData(NamedTuple):
    df: pd.DataFrame
    X: pd.DataFrame
    y: pd.DataFrame


@pytest.fixture
def sample_data():
    df = pd.DataFrame(
        data={
            'country': ['USA', 'USA', 'Canada', 'Japan'],
            'year': [2001, 2002, 2001, 2002],
            'size': [np.nan, -11, 12, 0.0],
            'strength': [18, 110, np.nan, 101],
            'happy': [False, True, False, False]
        }
    ).set_index(['country', 'year'])
    X = df[['size', 'strength']]
    y = df[['happy']]
    return SampleData(df, X, y)



import numpy as np
import pandas as pd
import pytest

from ballet.eng.misc import IdentityTransformer
from ballet.feature import Feature
from ballet.pipeline import FeatureEngineeringPipeline


@pytest.fixture
def inputs():
    input = 'foo'
    transformer = IdentityTransformer()
    return input, transformer


def test_init_seqcont(inputs):
    input, transformer = inputs
    feature = Feature(input, transformer)
    features = [feature]
    mapper = FeatureEngineeringPipeline(features)
    assert isinstance(mapper, FeatureEngineeringPipeline)


def test_init_scalar(inputs):
    input, transformer = inputs
    feature = Feature(input, transformer)
    mapper = FeatureEngineeringPipeline(feature)
    assert isinstance(mapper, FeatureEngineeringPipeline)


def test_fit(inputs):
    input, transformer = inputs
    feature = Feature(input, transformer)
    mapper = FeatureEngineeringPipeline(feature)
    df = pd.util.testing.makeCustomDataframe(5, 2)
    df.columns = ['foo', 'bar']
    mapper.fit(df)


def test_transform(inputs):
    input, transformer = inputs
    feature = Feature(input, transformer)
    mapper = FeatureEngineeringPipeline(feature)
    df = pd.util.testing.makeCustomDataframe(5, 2)
    df.columns = ['foo', 'bar']
    mapper.fit(df)
    X = mapper.transform(df)
    assert np.shape(X) == (5, 1)



import logging
from unittest.mock import Mock, patch

import git
import pytest

import ballet
import ballet.util.log
from ballet.update import (
    _check_for_updated_ballet, _extract_latest_from_search_triple,
    _get_latest_ballet_version_string, _log_recommended_reinstall,
    _make_template_branch_merge_commit_message, _query_pip_search_ballet,
    _safe_delete_remote, _warn_of_updated_ballet,)


@pytest.mark.skip(
    reason='Disabled due to https://status.python.org/incidents/grk0k7sz6zkp'
)
def test_query_pip_search_ballet():
    # nothing better to do that just call the function...
    # actually hits PyPI but difficult to mock because uses subprocess >:(
    result = _query_pip_search_ballet()
    assert 'ballet' in result


def test_extract_latest_from_search_triple():
    triple = (
        'ballet (x.y.z)  - some description',
        '  INSTALLED: x.y.z',
        '  LATEST:    u.v.w',
    )
    result = _extract_latest_from_search_triple(triple)
    assert result == 'u.v.w'


def test_extract_latest_from_search_triple_not_found():
    triple = (
        'something',
        'that does not',
        'match the expected format'
    )
    result = _extract_latest_from_search_triple(triple)
    assert result is None


@patch('funcy.partition')
@patch('ballet.update._extract_latest_from_search_triple')
@patch('ballet.update._query_pip_search_ballet')
def test_get_latest_ballet_version_string(
    mock_query, mock_extract, mock_partition
):
    mock_partition.return_value = ['some', 'iterable']
    expected = 'x.y.z'
    mock_extract.return_value = expected

    actual = _get_latest_ballet_version_string()

    assert actual == expected


@patch('ballet.update._get_latest_ballet_version_string')
def test_check_for_updated_ballet(mock_latest):
    # obviously this will represent an update from whatever the current
    # version is
    latest = '99999999999.0.0'
    mock_latest.return_value = latest
    expected = latest
    actual = _check_for_updated_ballet()
    assert actual == expected


@patch('ballet.update._get_latest_ballet_version_string')
def test_check_for_updated_ballet_no_updates(mock_latest):
    mock_latest.return_value = ballet.__version__
    expected = None  # no updates available
    actual = _check_for_updated_ballet()
    assert actual == expected


def test_warn_of_updated_ballet(caplog):
    caplog.set_level(logging.DEBUG, ballet.util.log.logger.name)
    latest = 'x.y.z'
    _warn_of_updated_ballet(latest)
    assert latest in caplog.text


def test_make_template_branch_merge_commit_message():
    result = _make_template_branch_merge_commit_message()
    assert ballet.__version__ in result


def test_safe_delete_remote():
    repo = Mock(spec=git.Repo)
    name = 'name'
    _safe_delete_remote(repo, name)
    repo.delete_remote.assert_called_once_with(name)


def test_log_recommended_reinstall(caplog):
    caplog.set_level(logging.DEBUG, ballet.util.log.logger.name)
    _log_recommended_reinstall()
    assert caplog.text
